package ch.il06.zeiterfassung.db;

import java.util.Date;

public interface DbInterface {
	/**
	 * Login-Funktion. Wenn die Daten richtig sind, wird ein User-Objekt
	 * zurückgegeben. Ansonsten null.
	 * 
	 * @return User|null
	 * @throws Exception 
	 */
	public User login(String pid, String password) throws Exception;

	/**
	 * Sucht einen oder mehrere User und gibt ein User-Objekt-Array zurück. Gibt
	 * null zurück, wenn nichts gefunden wurde.
	 * 
	 * @param pid
	 *            PID des gesuchten Benutzers
	 * @param name
	 *            Name
	 * @param firstname
	 *            Vorname
	 * @param group
	 *            Gruppe (z.B. IL06)
	 * @return User[]|null
	 * @throws Exception 
	 */
	public User[] searchUser(String pid, String name, String firstname,
			Group group) throws Exception;

	/**
	 * Sucht Gruppen, die den angegebenen String im Name hat. Gibt null zurück,
	 * wenn nichts gefunden wurde.
	 * 
	 * @param group Teilstring (z.B. IL, 06)
	 * @return Group[]|null
	 */
	public Group[] searchGroup(String group, User lehrmeister) throws Exception;

	/**
	 * Erstellt eine neue Gruppe.
	 * 
	 * @param name
	 *            Name der Gruppe
	 * @param master
	 *            Zuständiger Lehrmeister
	 * @return Erstellte Gruppe
	 * @throws Exception 
	 */
	public Group createGroup(String name, User master) throws Exception;

	/**
	 * Gibt den Kalender zu einem Lehrling zurück.
	 * 
	 * @param user
	 *            Lerhling
	 * @return Lehrlingskalender
	 */
	public UserCalendar getCalendarByApprentice(User user);

	/**
	 * Gibt den Kalender zu einer Gruppe/Klasse zurück.
	 * 
	 * @param group
	 *            Gruppe/Klasse
	 * @return Gruppenkalender
	 */
	public GroupCalendar getCalendarByGroup(Group group);

	/**
	 * Gibt den Feiertagskalender zurück.
	 * 
	 * @return Feiertagskalender
	 */
	public OfficialHolidayCalendar getOfficialHolidayCalendar();

	/**
	 * Gibt alle eingetragenen Tage zurück, die zwischen "from" und "to" liegen.
	 * Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return Day[]|null
	 * @depreceated Sollte nicht mehr benutzt werden. Day wurde durch DayRange ersetzt!
	 */
	@Deprecated
	public Day[] getDays(Date from, Date to, Calendar calendar);

	/**
	 * Gibt alle eingetragenen Ausbildungs-Tage zurück, die zwischen "from" und
	 * "to" liegen. Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return GroupDayRange[]|null
	 * @throws Exception 
	 */
	public DayRange[] getGroupDayRanges(Date from, Date to, GroupCalendar calendar) throws Exception;

	/**
	 * Gibt alle eingetragenen User-Tage (Arbeitszeit) zurück, die zwischen
	 * "from" und "to" liegen. Wenn keine Tage dazwischen liegen, wird null
	 * zurückgegeben.
	 * 
	 * @return WorkDay[]|null
	 * @throws Exception 
	 */
	public WorkDay[] getWorkDays(Date from, Date to, UserCalendar calendar) throws Exception;

	/**
	 * Gibt alle eingetragenen User-Ausfall-Tage (z.B. Krankheit ...) zurück,
	 * die zwischen "from" und "to" liegen. Wenn keine Tage dazwischen liegen,
	 * wird null zurückgegeben.
	 * 
	 * @return OutfallDayRange[]|null
	 * @throws Exception 
	 */
	public DayRange[] getOutfallDayRanges(Date from, Date to, UserCalendar calendar) throws Exception;

	/**
	 * Gibt alle eingetragenen Feiertage zurück, die zwischen "from" und "to"
	 * liegen. Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return OfficialHolidayRange[]|null
	 * @throws Exception 
	 */
	public DayRange[] getHolidayRanges(Date from, Date to,
			OfficialHolidayCalendar calendar) throws Exception;

	/**
	 * Gibt alle Lehrlinge zurück die, zum übergebenen Berufsbildner gehören.
	 * 
	 * @param instructor
	 *            Berufsbildner
	 * @return User[] Gefundene User
	 * @throws Exception 
	 */
	public User[] getUserByInstructor(User instructor) throws Exception;

	@Deprecated
	public void save(Day day, Calendar calendar);

	/**
	 * Einen Arbeitstag in den User-Kalender eintragen.
	 * @throws Exception 
	 */
	public void save(WorkDay day, UserCalendar calendar) throws Exception;

	/**
	 * Eine DayRange in einen Kalendar eintragen.
	 * @throws Exception 
	 */
	public void save(DayRange day, Calendar calendar) throws Exception;

	/**
	 * Visieren, Monats Start bis Monats Ende mitgeben.
	 * Datumsformat wie folgt: Date(2008-1900,10-1,01)
	 * @throws Exception 
	 */
	public void visieren(Date fromday, Date today, UserCalendar calendar) throws Exception;

	/**
	 * Legt einen neuen Benutzer an oder speichert Änderungen eines bestehenden.
	 * Gibt die generierte ID des Users zurück (diese wird auch im User-Objekt gesetzt).
	 * 
	 * @param User User der gespeichert werden soll
	 * @return Generierte ID
	 * @throws Exception 
	 */
	int save(User user) throws Exception;

	/**
	 * Legt eine neue Gruppe an oder speicher Änderungern einer bestehenden Gruppe.
	 * Gibt die generierte ID der Gruppe zurück (diese wird auch im Gruppen-Objekt gesetzt.
	 * 
	 * @param group Gruppe die gespeichert werden soll
	 * @return Generierte ID
	 * @throws Exception
	 */
	public int save(Group group) throws Exception;

	/**
	 * Gibt alle User einer bestimmten Rolle zurück.
	 * 
	 * @param role Rolle, nach welcher gesucht werden soll
	 * @return Gefundene User
	 * @throws Exception
	 */
	public User[] getUsersByRole(Role role) throws Exception;
}
