package ch.il06.zeiterfassung.web;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import ch.il06.zeiterfassung.web.exception.ControllerNotFoundException;

/**
 * Dies ist der Frontcontroller
 *
 * @author ncaspar
 */
public class IndexServlet extends HttpServlet {
    
    /**
	 * Standard Serial Version
	 */
	private static final long serialVersionUID = 1L;

    /** 
     * Verarbeitet den HTTP-<code>GET</code>-Request
     * 
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException wenn ein Fehler bei der Bearbeitung auftritt
     * @throws IOException wenn es ein Fehler bei der Ausgabe auftritt
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Verarbeitet den HTTP-<code>POST</code>-Request
     * 
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException wenn ein Fehler bei der Bearbeitung auftritt
     * @throws IOException wenn es ein Fehler bei der Ausgabe auftritt
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

	/** 
     * Verarbeitet sowohl <code>GET</code>- als auch <code>POST</code>-Requests
     * 
     * @param request servlet request
     * @param response servlet response
     * @throws IOException wenn es ein Fehler bei der Ausgabe auftritt
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        try {
            handleRequest(request, response);
        } catch (ControllerNotFoundException ex) {
            response.setStatus(404);
            String message = "FEHLER: " + ex.getMessage();
            printMessage(response, message);
        } catch (Exception ex) {
            response.setStatus(500);
            response.setContentType("text/plain");
            ex.printStackTrace(response.getWriter());
            response.getWriter().close();
            response.flushBuffer();
        }
    }

	/**
	 * Lädt den Controller, prüft die Rechte und ruft die Action auf.
	 * 
	 * @param request
	 * @param response
	 * @throws ControllerNotFoundException
	 * @throws IOException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ServletException
	 * @throws InvocationTargetException
	 */
	private void handleRequest(HttpServletRequest request,
			HttpServletResponse response) throws ControllerNotFoundException,
			IOException, InstantiationException, IllegalAccessException,
			ServletException, InvocationTargetException {
		response.setContentType("text/html;charset=UTF-8");
		response.setStatus(200);
		
		Class<? extends AbstractController> controllerClass = _loadController(request);

		String action = _loadAction(request);
		Method callAction = getMethodByAction(controllerClass, action);
		if (callAction == null) {
		    response.setStatus(404);
		    printMessage(response, "FEHLER: Action " + action + " not found ");
		    return;
		}

		AbstractController controller = (AbstractController) controllerClass.newInstance();
		controller.setVariables(request, response);
		if(!controller.isAllowed(action)) {
		    response.setStatus(403);
		    request.getRequestDispatcher("/index/index.jsp").forward(request, response);
		    return;
		}
		callAction.invoke(controller);
	}

	/**
	 * Sucht die zur Action passende Methode
	 * 
	 * @param controllerClass
	 * @param action
	 * @return
	 */
	private Method getMethodByAction(
			Class<? extends AbstractController> controllerClass, String action) {
		Method[] methods = controllerClass.getMethods();
		for (int i = 0; i < methods.length; i++) {
		    if (methods[i].getName().equals(action)) {
		        return methods[i];
		    }
		}
		return null;
	}

	/**
	 * Gibt eine Fehlermeldung im Browser aus.
	 * 
	 * @param response
	 * @param message
	 * @throws IOException
	 */
	private void printMessage(HttpServletResponse response, String message)
			throws IOException {
		response.setContentType("text/plain");
		response.getWriter().print(message);
		response.getWriter().close();
		response.flushBuffer();
	}

    /**
     * Gibt die URL zurück, die der User im Browser eingegeben hat.
     * 
     * @param request
     * @return url
     */
    private String _getUrl(HttpServletRequest request) {
        String url = "" + request.getAttribute("javax.servlet.error.request_uri");
        if(url == null || "".equals(url) || "null".equals(url)) {
            url = request.getContextPath() + "/";
        }
        return url;
    }

    /**
     * Lädt die passende Action
     * 
     * @param request
     * @return Name der Action
     */
    private String _loadAction(HttpServletRequest request) {
        String url = _getUrl(request);
        url = url.substring(request.getContextPath().length() + 1);
        int pos = url.indexOf("/");
        if (pos == -1) {
            pos = url.length() - 1;
        }
        pos += 1;
        String action = url.substring(pos);
        if ("".equals(action)) {
            action = "index";
        }
        action = action.toLowerCase();
        return action + "Action";
    }

    /**
     * Ladet die zur URL passende Java-Klasse (Controller)
     * 
     * @param request
     * @return Controller-Klasse als Class
     * @throws ControllerNotFoundException wenn der Controller nicht gefunden werden kann
     */
    private Class<? extends AbstractController> _loadController(HttpServletRequest request) throws ControllerNotFoundException {
        String url = _getUrl(request);
        url = url.substring(request.getContextPath().length() + 1);
        int pos = url.indexOf("/");
        if (pos == -1) {
            pos = url.length();
        }
        String controller = url.substring(0, pos);
        if ("".equals(controller)) {
            controller = "index";
        }
        controller = controller.substring(0, 1).toUpperCase() + controller.substring(1).toLowerCase();

        Class<? extends AbstractController> controllerClass = null;
        String controllerPath = "ch.il06.zeiterfassung.web.controller." + controller + "Controller";
        try {
            controllerClass = (Class<? extends AbstractController>) Class.forName(controllerPath).asSubclass(AbstractController.class);
        } catch (ClassNotFoundException ex) {
            throw new ControllerNotFoundException(controllerPath + " could no be loaded.");
        }
        return controllerClass;
    }

    /** 
     * Gibt eine kurze Beschreibung des Servlets zurück
     * 
     * @return Servlet-Beschreibung
     */
    @Override
    public String getServletInfo() {
        return "Frontcontroller";
    }
}
