package ch.il06.zeiterfassung.db;

import java.util.Date;

public interface DbInterface {
	/**
	 * Login-Funktion. Wenn die Daten richtig sind, wird ein User-Objekt
	 * zurückgegeben. Ansonsten null.
	 * 
	 * @return User|null
	 */
	public User login(String pid, String password);

	/**
	 * Sucht einen oder mehrere User und gibt ein User-Objekt-Array zurück. Gibt
	 * null zurück, wenn nichts gefunden wurde.
	 * 
	 * @param pid
	 *            PID des gesuchten Benutzers
	 * @param name
	 *            Name
	 * @param firstname
	 *            Vorname
	 * @param group
	 *            Gruppe (z.B. IL06)
	 * @return User[]|null
	 */
	public User[] searchUser(String pid, String name, String firstname,
			Group group);

	/**
	 * Sucht Gruppen, die den angegebenen String im Name hat. Gibt null zurück,
	 * wenn nichts gefunden wurde.
	 * 
	 * @param group
	 *            Teilstring (z.B. IL, 06)
	 * @return Group[]|null
	 */
	public Group[] searchGroup(String group);

	/**
	 * Erstellt eine neue Gruppe.
	 * 
	 * @param name
	 *            Name der Gruppe
	 * @param master
	 *            Zuständiger Lehrmeister
	 * @return Erstellte Gruppe
	 */
	public Group createGroup(String name, User master);

	/**
	 * Gibt den Kalender zu einem Lehrling zurück.
	 * 
	 * @param user
	 *            Lerhling
	 * @return Lehrlingskalender
	 */
	public UserCalendar getCalendarByApprentice(User user);

	/**
	 * Gibt den Kalender zu einer Gruppe/Klasse zurück.
	 * 
	 * @param group
	 *            Gruppe/Klasse
	 * @return Gruppenkalender
	 */
	public GroupCalendar getCalendarByGroup(Group group);

	/**
	 * Gibt den Feiertagskalender zurück.
	 * 
	 * @return Feiertagskalender
	 */
	public OfficialHolidayCalendar getOfficialHolidayCalendar();

	/**
	 * Gibt alle eingetragenen Tage zurück, die zwischen "from" und "to" liegen.
	 * Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return Day[]|null
	 * @depreceated Sollte nicht mehr benutzt werden.
	 */
	@Deprecated
	public Day[] getDays(Date from, Date to, Calendar calendar);

	/**
	 * Gibt alle eingetragenen Ausbildungs-Tage zurück, die zwischen "from" und
	 * "to" liegen. Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return GroupDayRange[]|null
	 */
	public GroupDayRange[] getGroupDayRanges(Date from, Date to, GroupCalendar calendar);

	/**
	 * Gibt alle eingetragenen User-Tage (Arbeitszeit) zurück, die zwischen
	 * "from" und "to" liegen. Wenn keine Tage dazwischen liegen, wird null
	 * zurückgegeben.
	 * 
	 * @return WorkDay[]|null
	 */
	public WorkDay[] getWorkDays(Date from, Date to, UserCalendar calendar);

	/**
	 * Gibt alle eingetragenen User-Ausfall-Tage (z.B. Krankheit ...) zurück,
	 * die zwischen "from" und "to" liegen. Wenn keine Tage dazwischen liegen,
	 * wird null zurückgegeben.
	 * 
	 * @return OutfallDayRange[]|null
	 */
	public OutfallDayRange[] getOutfallDayRanges(Date from, Date to, UserCalendar calendar);

	/**
	 * Gibt alle eingetragenen Feiertage zurück, die zwischen "from" und "to"
	 * liegen. Wenn keine Tage dazwischen liegen, wird null zurückgegeben.
	 * 
	 * @return OfficialHolidayRange[]|null
	 */
	public OfficialHolidayRange[] getHolidayRanges(Date from, Date to,
			OfficialHolidayCalendar calendar);

	/**
	 * Gibt alle Lehrlinge zurück die, zum übergebenen Berufsbildner gehören.
	 * 
	 * @param instructor
	 *            Berufsbildner
	 * @return User[] Gefundene User
	 */
	public User[] getUserByInstructor(User instructor);

	@Deprecated
	public void save(Day day, Calendar calendar);

	/**
	 * Einen Arbeitstag in den User-Kalender eintragen.
	 */
	public void save(WorkDay day, UserCalendar calendar);

	/**
	 * Einen Ausfalltag in den User-Kalender eintragen.
	 */
	public void save(OutfallDayRange day, UserCalendar calendar);

	/**
	 * Einen Ausbildungstag in den Gruppen-Kalender eintragen.
	 */
	public void save(GroupDayRange day, GroupCalendar calendar);

	/**
	 * Visieren, Monats Start bis Monats Ende mitgeben.
	 * Datumsformat wie folgt: Date(2008-1900,10-1,01)
	 */
	public void visieren(Date fromday, Date today, UserCalendar calendar);
	
	/**
	 * Einen Feiertag in den Feiertags-Kalender eintragen.
	 */
	public void save(OfficialHolidayRange day, OfficialHolidayCalendar calendar);

	/**
	 * Legt einen neuen Benutzer an oder speichert Änderungen eines bestehenden.
	 * Gibt die generierte ID des Users zurück.
	 * 
	 * @param User
	 *            User der gespeichert werden soll
	 * @return Generierte ID
	 */
	int save(User user);

	public int save(Group group);



}
